/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Novell Inc      
 *	The copyright notice above does not evidence any   	      
 *	actual or intended publication of such source code.	       
 */

# /* ident	"$Id: slp.h,v 1.3 1998/04/24 09:54:18 prashant Exp $" */
/*
 * Copyright 1994 Unpublished Work of Novell, Inc.
 * All Rights Reserved.
 *
 * This work is an unpublished work and contains confidential,
 * proprietary and trade secret information of Novell, Inc. Access
 * to this work is restricted to (I) Novell employees who have a
 * need to know to perform tasks within the scope of their
 * assignments and (II) entities other than Novell who have
 * entered into appropriate agreements.
 *
 * No part of this work may be used, practiced, performed,
 * copied, distributed, revised, modified, translated, abridged,
 * condensed, expanded, collected, compiled, linked, recast,
 * transformed or adapted without the prior written consent
 * of Novell.  Any use or exploitation of this work without
 * authorization could subject the perpetrator to criminal and
 * civil liability.
 */
#include <netinet/in.h>

#ifndef SLP_H
   #define SLP_H

#ifdef __cplusplus
   extern "C" {
#endif                                 /* for C++ use of C interface */

#define   EXPORT
typedef enum  
{
     SL_LIFETIME_NONE = 0,             /*   register indefinitely   */
     SL_LIFETIME_DEFAULT = 0xFFFF     /*   use CONFIG_INTERVAL_1   */
} SL_SpecialLifeTime;

/**  There is no enumeration type for unicode so we define here.  **/
typedef enum 
{
     SL_CHAR_ASCII = 3, SL_CHAR_LATIN1 = 4, SL_CHAR_UTF8 = 106, 
     SL_CHAR_UNICODE = 1000
} SL_CharType;


typedef enum 
{
   SL_TYPE_RESULT = 1, SL_ATTR_RESULT = 2, SL_SRV_RESULT = 3,
   SL_SCOPE_RESULT = 4
} SL_ResType;


typedef enum 
{
   SL_STR_VAL = 1, SL_INT_VAL = 2,  SL_OPAQUE_VAL = 3, SL_BOOL_VAL = 4
} SL_ValType;



/*
      SLP errors, can be received from a DA or SA.
*/

typedef enum 
{
     SLP_OK                     = 0,
     SLP_LANG_NOT_SUPPORTED     = 0x01,
     SLP_PROTOCOL_PARSE_ERR     = 0x02,
     SLP_INVALID_REGISTRATION   = 0x03,
     SLP_SCOPE_NOT_SUPPORTED    = 0x04,
     SLP_CHARSET_NOT_UNDERSTOOD = 0x05,
     SLP_AUTHENTICATION_INVALID = 0x06,
     /*
           Interface errors.
      */
     SLP_NOT_SUPPORTED_YET            = 0x10,
     SLP_REQUEST_TIMED_OUT            = 0x11,
     SLP_COULD_NOT_INIT_NET_RESOURCES = 0x12,
     SLP_COULD_NOT_ALLOCATE_MEMORY    = 0x13,
     SLP_PARAMETER_BAD                = 0x14,
     SLP_INTERNAL_NET_ERROR           = 0x15,
     SLP_INTERNAL_SYSTEM_ERROR        = 0x16
} SL_Error;

typedef enum 
{
     SL_Srv_IgnoreCache    = 0x01, /* Will ignore UA caches            */
     SL_Srv_DoNotResolve   = 0x02, /* Will not resolve URL domain name */
                                   /* to struct sockaddr_in            */
     SL_Srv_ForceMulticast = 0x04  /* Will not use DA, will multicast  */
} SL_SrvFlags;



/*
     All ENUMERATIONS done. Now typedefs.
*/

typedef struct servicetag 
{
     char               *s_pcURL;
     struct sockaddr_in  s_sin;
     int                 s_iLifetime;
} SL_Service;



typedef struct opaquetag 
{
     int      op_iLen;
     char    *op_pcVal;
} SL_Opaque;



typedef union 
{
     long      intval;
     int       boolval;
     SL_Opaque opaqueval;
     char     *pcStrVal;
} SL_Value;

typedef struct attrtag 
{
     int         a_iNumVals;
     char       *a_pcAttrTag;
     SL_ValType  a_slvaltype;
     SL_Value   *a_pslValue;
} SL_Attribute;

typedef union 
{
     SL_Service    i_sl_service;
     SL_Attribute  i_sl_attribute;
     char         *i_pcSrvtype;
     char         *i_pcScope;
} SL_ResInfo;

typedef struct resulttag 
{
     SL_Error    r_iErrorCode;
     int         r_iNumValues;
     int         r_iCharEncoding;
     char       *r_pcLang;
     SL_ResType  r_sl_restype;
     SL_ResInfo *r_psl_Info;
} SL_Result;



typedef void* SL_SrvHandle;


/*
    * Ease-of-use macros, to enable dereferencing of structures.
    *
    *    res_srv_url        returns the URL of the i'th service
    *                          in the service result
    *    res_srv_sin        returns the struct sockaddr_in of the
    *                          i'th service in the service result
    *    res_srv_life       returns the lifetime in seconds of the
    *                          i'th service in the service result
    *    res_srvtype        returns the string of the i'th service
    *                          type in the service type result
    *    res_scope          returns the scope string of the i'th
    *                          scope result
    *    res_attr_numval    returns the number of attr values for
    *                          the i'th attr in the attr result.
    *    res_attr_tag       returns the tag string for the i'th
    *                          attr in the attr result
    *    res_attr_type      returns the type for the i'th attr
    *                          in the attr result
    *    res_attr_str       returns the j'th string value of the
    *                          i'th attr in the attr result
    *    res_attr_int       returns the j'th int value of the i'th
    *                          attr in the attr_result
    *    res_attr_bool      returns the j'th boolean value of the
    *                          i'th attr in the attr result
    *    res_attr_opaque    returns the j'th opaque value of the
    *                          i'th attr in the attr result
*/

#define res_srv_url(i)     r_psl_Info[(i)].i_sl_service.s_pcURL
#define res_srv_sin(i)     r_psl_Info[(i)].i_sl_service.s_sin
#define res_srv_life(i)    r_psl_Info[(i)].i_sl_service.s_iLifetime
#define res_srvtype(i)     r_psl_Info[(i)].i_pcSrvtype
#define res_scope(i)       r_psl_Info[(i)].i_pcScope
#define res_attr_numval(i) r_psl_Info[(i)].i_sl_attribute.a_iNumVals
#define res_attr_tag(i)    r_psl_Info[(i)].i_sl_attribute.a_pcAttrTag
#define res_attr_type(i)   r_psl_Info[(i)].i_sl_attribute.a_slvaltype
#define res_attr_str(i,j)  r_psl_Info[(i)].i_sl_attribute.a_pslValue[(j)].pcStrVal
#define res_attr_int(i,j)  r_psl_Info[(i)].i_sl_attribute.a_pslValue[(j)].intval
#define res_attr_bool(i,j) r_psl_Info[(i)].i_sl_attribute.a_pslValue[(j)].boolval
#define res_attr_opaque(i,j) r_psl_Info[(i)].i_sl_attribute.a_pslValue[(j)].opaqueval


/*
    * General interfaces for all clients
    *
    * The term "EXPORT" is included so as to make the interface portable
    * to Win32.  For Unix environments, etc. EXPORT is defined as nothing.
    * For Win32, EXPORT is __declspec(dllimport) for the client library
    * and __declspec(dllexport) for the library implementation.
*/


/*------------------------------------------------------------------*/
EXPORT SL_SrvHandle SL_Open(
            SL_SrvHandle hSL_SrvHandle,    /* State created by SL_Open   */
            const char  *pcLangPref,       /* Set to the ISO 639 code    */
            int          iDialect,         /* Set to 0, reserved         */
            int          iMonoling,        /* 1 = Monolingual, 0 = not   */
            int          iMaxWait,         /* # seconds max, 0 = default */
            int          iMultiple,        /* 0 = return first result,   */
                                           /* 1 = return all results     */
            SL_CharType  slchartype,       /* Type for characters used.  */
            SL_Error    *pErrValue  );     /* 0 unless an error occurred */

/*------------------------------------------------------------------*/

EXPORT void  SL_Close(
             SL_SrvHandle *pSL_SrvHandle  ); /* pSrvState from SL_Open  */

/*------------------------------------------------------------------*/



/*------------------------------------------------------------------
    * SA interfaces
*/



/*------------------------------------------------------------------*/
EXPORT SL_Error SL_Register(
            SL_SrvHandle hSL_SrvHandle,    /* State created by SL_Open  */
            const char  *pcURL,            /* Unique URL (of reg.)      */
            const char  *pcLang,           /* Language code of reg.     */
            int          iDialect,         /* Set to 0, reserved.       */
            SL_CharType  slchartype,       /* Character type of reg.    */
            const char  *pcAttributes,     /* Service Attribute list    */
            int          iLifetime  );     /* 1..65534 = # of seconds   */


/*------------------------------------------------------------------*/
EXPORT SL_Error SL_DeRegister(
            SL_SrvHandle hSL_SrvHandle,    /* State created by SL_Open  */
            const char  *pcURL,            /* URL (of service todereg.) */
            const char  *pcLang,           /* Language code of dereg.   */
            int          iDialect,         /* Set to 0, reserved.       */
            SL_CharType  slchartype,       /* Character type of dereg.  */
            const char  *pcAttrTags  );    /* Tags of attrs to dereg.   */


/*------------------------------------------------------------------*/
EXPORT SL_Error SL_Binary2Opaque(
         const SL_Opaque *pslo,           /* Opaque to convert         */
         char       *pcOpaqueBuffer,      /* User supplied data buffer */
         int         iLenOpaqueBuffer  ); /* Length of data buffer     */


/*------------------------------------------------------------------*/


/*------------------------------------------------------------------
     *  UA interfaces
*/



/*------------------------------------------------------------------*/

EXPORT SL_Result * SL_GetScopes(
            SL_SrvHandle hSL_SrvHandle  ); /* State created by SL_Open  */

/*------------------------------------------------------------------*/

EXPORT SL_Result * SL_GetSrvTypes(
            SL_SrvHandle hSL_SrvHandle,    /* State created by SL_Open  */
            const char  *pcNamingAuth,     /* NULL = default            */
            const char  *pcScope  );       /* NULL = none               */

/*------------------------------------------------------------------*/

EXPORT SL_Result * SL_GetAttrs(
            SL_SrvHandle hSL_SrvHandle,    /* State created by SL_Open  */
            const char  *pcURL,            /* Either full or partial    */
            const char  *pcScope,          /* Scope of attrs to get     */
            const char  *pcAttrTags  );    /* Tags for 'selected' attrs */

/*------------------------------------------------------------------*/

EXPORT SL_Result * SL_GetService(
            SL_SrvHandle hSL_SrvHandle,    /* State created by SL_Open  */
            const char  *pcRequest,        /* Request string to supply  */
            int          iFlags  );        /* 0 = none, otherwise '|'   */
                                           /* 'SL_Srv' flags, see above */

/*------------------------------------------------------------------*/

EXPORT void    SL_FreeResult(
            SL_SrvHandle hSL_SrvHandle,    /* State created by SL_Open  */
            SL_Result  **ppResult  );     /* SL_Result to be freed     */

/*
  My declaration and definitions.
*/

extern   const int  iSL_PermSrvHandle;
#define  SL_PERM_SRV_HANDLE  (void *) &iSL_PermSrvHandle

#ifdef __cplusplus
   }
#endif /* for C++ use of C interface */

#endif /* End of the SLP API definition */


/*------------------------------------------------------------------*/
